@@ -14,18 +14,28 @@ Key Features:
 - Doctests utilities
 
 """
+# ruff: noqa: F403
 
 # Imports
-from .print import *
-from .archive import *
-from .io import *
-from .decorators import *
-from .ctx import *
-from .parallel import *
 from .all_doctests import *
-from .collections import *
+from .archive import *
 from .backup import *
+from .collections import *
 
 # Folders
 from .continuous_delivery import *
+from .ctx import *
+from .decorators import *
+from .image import *
+from .io import *
+from .parallel import *
+from .print import *
+
+# Version (handle case where the package is not installed)
+import importlib.metadata
+
+try:
+	__version__: str = importlib.metadata.version("stouputils")
+except importlib.metadata.PackageNotFoundError:
+	__version__: str = "0.0.0-dev"
 
