@@ -2,14 +2,21 @@
 
 This module provides higher-level utilities for specific application needs:
 
-Key Features:
+Automatic Documentation:
+
 - Automatic documentation generation with Sphinx: `update_documentation(...)`
 - Support for multi-version documentation
 - GitHub Pages integration
 - Markdown to RST conversion
 
+Upscaler:
+
+- TODO
+
 """
+# ruff: noqa: F403
 
 # Imports
 from .automatic_docs import *
+from .upscaler import *
 
