@@ -1,19 +1,22 @@
 """ This module contains utilities for continuous delivery on GitHub.
 
-- upload_to_github: Upload the project to GitHub using the credentials and the configuration (make a release and upload the assets, handle existing tag, generate changelog, etc.)
+- upload_to_github: Upload the project to GitHub using the credentials and the configuration
+  (make a release and upload the assets, handle existing tag, generate changelog, etc.)
 
 .. image:: https://raw.githubusercontent.com/Stoupy51/stouputils/refs/heads/main/assets/continuous_delivery/github_module.gif
   :alt: stouputils upload_to_github examples
 """
 
 # Imports
-from ..print import info, warning, progress
-from ..decorators import measure_time, handle_error
-from ..io import clean_path
-from .cd_utils import handle_response, version_to_float, clean_version
-from typing import Any
-import requests
 import os
+from typing import Any
+
+import requests
+
+from ..decorators import handle_error, measure_time
+from ..io import clean_path
+from ..print import info, progress, warning
+from .cd_utils import clean_version, handle_response, version_to_float
 
 # Constants
 GITHUB_API_URL: str = "https://api.github.com"
@@ -41,16 +44,28 @@ def validate_credentials(credentials: dict[str, dict[str, str]]) -> tuple[str, d
 	Returns:
 		tuple[str, dict[str, str]]:
 			str:			Owner (the username of the account to use)
-			
+
 			dict[str, str]:	Headers (for the requests to the GitHub API)
 	"""
 	if "github" not in credentials:
-		raise ValueError("The credentials file must contain a 'github' key, which is a dictionary containing a 'api_key' key (a PAT for the GitHub API: https://github.com/settings/tokens) and a 'username' key (the username of the account to use)")
+		raise ValueError(
+			"The credentials file must contain a 'github' key, which is a dictionary containing a 'api_key' key"
+			"(a PAT for the GitHub API: https://github.com/settings/tokens) "
+			"and a 'username' key (the username of the account to use)"
+		)
 	if "api_key" not in credentials["github"]:
-		raise ValueError("The credentials file must contain a 'github' key, which is a dictionary containing a 'api_key' key (a PAT for the GitHub API: https://github.com/settings/tokens) and a 'username' key (the username of the account to use)")
+		raise ValueError(
+			"The credentials file must contain a 'github' key, which is a dictionary containing a 'api_key' key"
+			"(a PAT for the GitHub API: https://github.com/settings/tokens) "
+			"and a 'username' key (the username of the account to use)"
+		)
 	if "username" not in credentials["github"]:
-		raise ValueError("The credentials file must contain a 'github' key, which is a dictionary containing a 'api_key' key (a PAT for the GitHub API: https://github.com/settings/tokens) and a 'username' key (the username of the account to use)")
-	
+		raise ValueError(
+			"The credentials file must contain a 'github' key, which is a dictionary containing a 'api_key' key"
+			"(a PAT for the GitHub API: https://github.com/settings/tokens) "
+			"and a 'username' key (the username of the account to use)"
+		)
+
 	api_key: str = credentials["github"]["api_key"]
 	owner: str = credentials["github"]["username"]
 	headers: dict[str, str] = {"Authorization": f"Bearer {api_key}"}
@@ -64,21 +79,36 @@ def validate_config(github_config: dict[str, Any]) -> tuple[str, str, str, list[
 	Returns:
 		tuple[str, str, str, list[str]]:
 			str: Project name on GitHub
-			
+
 			str: Version of the project
-			
+
 			str: Build folder path containing zip files to upload to the release
-			
+
 			list[str]: List of zip files to upload to the release
 	"""
 	if "project_name" not in github_config:
-		raise ValueError("The github_config file must contain a 'project_name' key, which is the name of the project on GitHub")
+		raise ValueError(
+			"The github_config file must contain a 'project_name' key, "
+			"which is the name of the project on GitHub"
+		)
 	if "version" not in github_config:
-		raise ValueError("The github_config file must contain a 'version' key, which is the version of the project")
+		raise ValueError(
+			"The github_config file must contain a 'version' key, "
+			"which is the version of the project"
+		)
 	if "build_folder" not in github_config:
-		raise ValueError("The github_config file must contain a 'build_folder' key, which is the folder containing the build of the project (datapack and resourcepack zip files)")
+		raise ValueError(
+			"The github_config file must contain a 'build_folder' key, "
+			"which is the folder containing the build of the project "
+			"(datapack and resourcepack zip files)"
+		)
 
-	return github_config["project_name"], github_config["version"], github_config["build_folder"], github_config.get("endswith", [])
+	project_name: str = github_config["project_name"]
+	version: str = github_config["version"]
+	build_folder: str = github_config["build_folder"]
+	endswith: list[str] = github_config.get("endswith", [])
+
+	return project_name, version, build_folder, endswith
 
 def handle_existing_tag(owner: str, project_name: str, version: str, headers: dict[str, str]) -> bool:
 	""" Check if tag exists and handle deletion if needed
@@ -140,7 +170,9 @@ def delete_existing_tag(tag_url: str, headers: dict[str, str]) -> None:
 	handle_response(delete_response, "Failed to delete existing tag")
 	info("Deleted existing tag")
 
-def get_latest_tag(owner: str, project_name: str, version: str, headers: dict[str, str]) -> tuple[str, str] | tuple[None, None]:
+def get_latest_tag(
+	owner: str, project_name: str, version: str, headers: dict[str, str]
+) -> tuple[str, str] | tuple[None, None]:
 	""" Get latest tag information
 
 	Args:
@@ -168,7 +200,9 @@ def get_latest_tag(owner: str, project_name: str, version: str, headers: dict[st
 	else:
 		return tags[0]["commit"]["sha"], clean_version(tags[0]["name"], keep="ab")
 
-def get_commits_since_tag(owner: str, project_name: str, latest_tag_sha: str|None, headers: dict[str, str]) -> list[dict[str, Any]]:
+def get_commits_since_tag(
+	owner: str, project_name: str, latest_tag_sha: str|None, headers: dict[str, str]
+) -> list[dict[str, Any]]:
 	""" Get commits since last tag
 
 	Args:
@@ -182,10 +216,10 @@ def get_commits_since_tag(owner: str, project_name: str, latest_tag_sha: str|Non
 	# Get the commits URL and parameters
 	commits_url: str = f"{PROJECT_ENDPOINT}/{owner}/{project_name}/commits"
 	commits_params: dict[str, str] = {"per_page": "100"}
-	
+
 	# Initialize tag_date as None
 	tag_date: str|None = None	# type: ignore
-	
+
 	# If there is a latest tag, use it to get the commits since the tag date
 	if latest_tag_sha:
 
@@ -194,10 +228,10 @@ def get_commits_since_tag(owner: str, project_name: str, latest_tag_sha: str|Non
 		tag_response = requests.get(tag_commit_url, headers=headers)
 		handle_response(tag_response, "Failed to get tag commit")
 		tag_date: str = tag_response.json()["commit"]["committer"]["date"]
-		
+
 		# Use the date as the 'since' parameter to get all commits after that date
 		commits_params["since"] = tag_date
-	
+
 	# Get the commits
 	response = requests.get(commits_url, headers=headers, params=commits_params)
 	handle_response(response, "Failed to get commits")
@@ -208,10 +242,12 @@ def get_commits_since_tag(owner: str, project_name: str, latest_tag_sha: str|Non
 		commits = [c for c in commits if c["commit"]["committer"]["date"] != tag_date]
 	return commits
 
-def generate_changelog(commits: list[dict[str, Any]], owner: str, project_name: str, latest_tag_version: str|None, version: str) -> str:
+def generate_changelog(
+	commits: list[dict[str, Any]], owner: str, project_name: str, latest_tag_version: str|None, version: str
+) -> str:
 	""" Generate changelog from commits. They must follow the conventional commits convention.
-	
-	Convention format: <type>: <description>
+
+	Convention format: <type>: <description> or <type>(<sub-category>): <description>
 
 	Args:
 		commits				(list[dict]):	List of commits to generate changelog from
@@ -225,42 +261,80 @@ def generate_changelog(commits: list[dict[str, Any]], owner: str, project_name:
 		https://www.conventionalcommits.org/en/v1.0.0/
 	"""
 	# Initialize the commit groups
-	commit_groups: dict[str, list[tuple[str, str]]] = {}
-	
+	commit_groups: dict[str, list[tuple[str, str, str | None]]] = {}
+
 	# Iterate over the commits
 	for commit in commits:
 		message: str = commit["commit"]["message"].split("\n")[0]
 		sha: str = commit["sha"]
-		
+
 		# If the message contains a colon, split the message into a type and a description
 		if ":" in message:
-			type_, desc = message.split(":", 1)
-			
-			# Clean the type
-			type_ = type_.split('(')[0]
-			type_ = "".join(c for c in type_.lower().strip() if c in "abcdefghijklmnopqrstuvwxyz")
-			type_ = COMMIT_TYPES.get(type_, type_.title())
-			
+			commit_type_part, desc = message.split(":", 1)
+
+			# Extract sub-category if present (e.g., 'feat(Project)' -> 'feat', 'Project')
+			sub_category: str|None = None
+			if "(" in commit_type_part and ")" in commit_type_part:
+				# Extract the base type (before parentheses)
+				commit_type: str = commit_type_part.split('(')[0].split('/')[0]
+				# Extract the sub-category (between parentheses)
+				sub_category = commit_type_part.split('(')[1].split(')')[0]
+			else:
+				# No sub-category, just clean the type
+				commit_type: str = commit_type_part.split('/')[0]
+
+			# Clean the type to only keep letters
+			commit_type = "".join(c for c in commit_type.lower().strip() if c in "abcdefghijklmnopqrstuvwxyz")
+			commit_type = COMMIT_TYPES.get(commit_type, commit_type.title())
+
 			# Add the commit to the commit groups
-			if type_ not in commit_groups:
-				commit_groups[type_] = []
-			commit_groups[type_].append((desc.strip(), sha))
+			if commit_type not in commit_groups:
+				commit_groups[commit_type] = []
+			commit_groups[commit_type].append((desc.strip(), sha, sub_category))
 
 	# Initialize the changelog
 	changelog: str = "## Changelog\n\n"
-	
-	# Iterate over the commit groups
-	for type_ in sorted(commit_groups.keys()):
-		changelog += f"### {type_}\n"
 
-		# Reverse the list to display the most recent commits in last
-		for desc, sha in commit_groups[type_][::-1]:
-			changelog += f"- {desc} ([{sha[:7]}](https://github.com/{owner}/{project_name}/commit/{sha}))\n"
+	# Iterate over the commit groups
+	for commit_type in sorted(commit_groups.keys()):
+		changelog += f"### {commit_type}\n"
+
+		# Group commits by sub-category
+		sub_category_groups: dict[str|None, list[tuple[str, str, str|None]]] = {}
+		for desc, sha, sub_category in commit_groups[commit_type]:
+			if sub_category not in sub_category_groups:
+				sub_category_groups[sub_category] = []
+			sub_category_groups[sub_category].append((desc, sha, sub_category))
+
+		# Sort sub-categories (None comes first, then alphabetical)
+		sorted_sub_categories = sorted(
+			sub_category_groups.keys(),
+			key=lambda x: (x is None, x or "")
+		)
+
+		# Iterate over sub-categories
+		for sub_category in sorted_sub_categories:
+
+			# Add commits for this sub-category
+			for desc, sha, _ in reversed(sub_category_groups[sub_category]):
+
+				# Prepend sub-category to description if present
+				if sub_category:
+					words: list[str] = [
+						word[0].upper() + word[1:] # We don't use title() because we don't want to lowercase any letter
+						for word in sub_category.replace('_', ' ').split()
+					]
+					formatted_sub_category: str = ' '.join(words)
+					formatted_desc = f"[{formatted_sub_category}] {desc}"
+				else:
+					formatted_desc = desc
+				changelog += f"- {formatted_desc} ([{sha[:7]}](https://github.com/{owner}/{project_name}/commit/{sha}))\n"
+
 		changelog += "\n"
-	
+
 	# Add the full changelog link if there is a latest tag and return the changelog
 	if latest_tag_version:
-		changelog += f"**Full Changelog**: https://github.com/{owner}/{project_name}/compare/v{latest_tag_version}...v{version}\n"	
+		changelog += f"**Full Changelog**: https://github.com/{owner}/{project_name}/compare/v{latest_tag_version}...v{version}\n"
 	return changelog
 
 def create_tag(owner: str, project_name: str, version: str, headers: dict[str, str]) -> None:
@@ -276,12 +350,12 @@ def create_tag(owner: str, project_name: str, version: str, headers: dict[str, s
 	progress(f"Creating tag v{version}")
 	create_tag_url: str = f"{PROJECT_ENDPOINT}/{owner}/{project_name}/git/refs"
 	latest_commit_url: str = f"{PROJECT_ENDPOINT}/{owner}/{project_name}/git/refs/heads/main"
-	
+
 	# Get the latest commit SHA
 	commit_response: requests.Response = requests.get(latest_commit_url, headers=headers)
 	handle_response(commit_response, "Failed to get latest commit")
 	commit_sha: str = commit_response.json()["object"]["sha"]
-	
+
 	# Create the tag
 	tag_data: dict[str, str] = {
 		"ref": f"refs/tags/v{version}",
@@ -312,13 +386,15 @@ def create_release(owner: str, project_name: str, version: str, changelog: str,
 		"draft": False,
 		"prerelease": False
 	}
-	
+
 	# Create the release and return the release ID
 	response: requests.Response = requests.post(release_url, headers=headers, json=release_data)
 	handle_response(response, "Failed to create release")
 	return response.json()["id"]
 
-def upload_assets(owner: str, project_name: str, release_id: int, build_folder: str, headers: dict[str, str], endswith: list[str]) -> None:
+def upload_assets(
+	owner: str, project_name: str, release_id: int, build_folder: str, headers: dict[str, str], endswith: list[str]
+) -> None:
 	""" Upload release assets
 
 	Args:
@@ -327,7 +403,8 @@ def upload_assets(owner: str, project_name: str, release_id: int, build_folder:
 		release_id		(int):				ID of the release to upload assets to
 		build_folder	(str):				Folder containing assets to upload
 		headers			(dict[str, str]):	Headers for GitHub API requests
-		endswith		(list[str]):		List of files to upload to the release (every file ending with one of these strings will be uploaded)
+		endswith		(list[str]):		List of files to upload to the release
+			(every file ending with one of these strings will be uploaded)
 	"""
 	endswith_tuple: tuple[str, ...] = tuple(endswith)
 
@@ -335,13 +412,14 @@ def upload_assets(owner: str, project_name: str, release_id: int, build_folder:
 	if not build_folder:
 		return
 	progress("Uploading assets")
-	
+
 	# Get the release details
-	response: requests.Response = requests.get(f"{PROJECT_ENDPOINT}/{owner}/{project_name}/releases/{release_id}", headers=headers)
+	release_url: str = f"{PROJECT_ENDPOINT}/{owner}/{project_name}/releases/{release_id}"
+	response: requests.Response = requests.get(release_url, headers=headers)
 	handle_response(response, "Failed to get release details")
 	upload_url_template: str = response.json()["upload_url"]
 	upload_url_base: str = upload_url_template.split("{", maxsplit=1)[0]
-	
+
 	# Iterate over the files in the build folder
 	for file in os.listdir(build_folder):
 		if file.endswith(endswith_tuple):
@@ -354,7 +432,7 @@ def upload_assets(owner: str, project_name: str, release_id: int, build_folder:
 					"Content-Type": "application/zip"
 				}
 				params: dict[str, str] = {"name": file}
-				
+
 				# Upload the file
 				response: requests.Response = requests.post(
 					upload_url_base,
@@ -400,17 +478,17 @@ def upload_to_github(credentials: dict[str, Any], github_config: dict[str, Any])
 
 	# Handle existing tag
 	can_create: bool = handle_existing_tag(owner, project_name, version, headers)
-	
+
 	# Get the latest tag and commits since the tag
 	latest_tag_sha, latest_tag_version = get_latest_tag(owner, project_name, version, headers)
 	commits: list[dict[str, Any]] = get_commits_since_tag(owner, project_name, latest_tag_sha, headers)
 	changelog: str = generate_changelog(commits, owner, project_name, latest_tag_version, version)
-	
+
 	# Create the tag and release if needed
 	if can_create:
 		create_tag(owner, project_name, version, headers)
 		release_id: int = create_release(owner, project_name, version, changelog, headers)
-		upload_assets(owner, project_name, release_id, build_folder, headers, endswith)	
+		upload_assets(owner, project_name, release_id, build_folder, headers, endswith)
 		info(f"Project '{project_name}' updated on GitHub!")
 	return changelog
 
